/*
 * Solves travelling salesman problem (TSP) with static & dynamic exemplary graphs
 */
 
import java.io.*;
import java.util.*;

public class TSP {
	private final static int INF = Integer.MAX_VALUE; // infinite distance
	private int numNodes = 3;				          // number of nodes in graph
	// matrix representation of distance from node_i to node_j
	// uses non-equidistant graph for somewhat more interesting results
	private int[][] distance = {{0, 3, 7},
						        {1, 0, 4},
						        {2, 6, 0}};
	
	// Calculate static example with 3 nodes and 6 paths
	// Returns shortest path as permutation with sequence of node indices
	public Permutation shortestPathStatic() {
		int minLength = INF;			                // initial minimal length
		int numPaths = 6;				                // number of different paths
		Permutation spath = null;                       // shortest path
		Permutation[] paths = new Permutation[numPaths];// array of paths
		// Enter all possible permutations of node indices => 3! different paths
		for (int k = 0; k < numPaths; k++)
			paths[k] = new Permutation();
		paths[0].add(0); paths[0].add(1); paths[0].add(2);
		paths[1].add(0); paths[1].add(2); paths[1].add(1);
		paths[2].add(1); paths[2].add(0); paths[2].add(2);
		paths[3].add(1); paths[3].add(2); paths[3].add(0);
		paths[4].add(2); paths[4].add(0); paths[4].add(1);
		paths[5].add(2); paths[5].add(1); paths[5].add(0);
		for (int i = 0; i < numPaths; i++) {
			// Determine length of current path and compare with previously found minimal length
			int length = pathLength(paths[i]);
			System.out.println("Checking path " + paths[i] + " with length " + length);
			if (length < minLength) {
				// Remember new minimal length and copy path
				minLength = length;
				spath = paths[i];
			}
		}		
		return spath;
	}

	// Calculate shortest path for arbitrary graphs (not suitable for very large graphs!)
	// Returns shortest path as integer array with sequence of node indices
	public Permutation shortestPath() {
		Permutation spath = null;
		int minLength = INF;
		// Use generator to determine all possible permutations of node indices
		PermutationGenerator gen = new PermutationGenerator();
		Set permutations = gen.generate(numNodes);
		Iterator it = permutations.iterator();
		while (it.hasNext()) {
			// Compare length of path to current minimal length
			Permutation path = (Permutation) it.next();
			int length = pathLength(path);
			System.out.println("Checking path " + path + " with length " + length);
			if (length < minLength) {
				minLength = length;
				spath = path;
			}
		}		
		return spath;
	}
	
	// Calculate length for given path represented as permutation of node indices
	private int pathLength(Permutation path) {
		int result = 0;
		for (int i = 0; i < numNodes; i++) {
			// Account for wrap-around from last node to first node in path
			int from = path.get(i);
			int to = path.get((i+1) % numNodes);
			result += distance[from][to];
		}
		return result;
	}
	
	// Read graph from text file with number of nodes and distances in format
	// line 1:   n
	// line 2:   d_00 d_01 ... d_0n
	// line n+1: d_n0 d_n1 ... d_nn
	public void readGraph(String fileName) {
		String buffer = new String();
		int row = 0;
		numNodes = 0;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(fileName));
			while (reader.ready()) {
				buffer = reader.readLine();
				if (numNodes == 0) {
					numNodes = Integer.parseInt(buffer);
					distance = new int[numNodes][numNodes];	
				} else {
					int column = 0;
					StringTokenizer tokenizer = new StringTokenizer(buffer);
					while (tokenizer.hasMoreTokens()) {
						String tmp = tokenizer.nextToken();
						distance[row][column] = Integer.parseInt(tmp);
						column++;
					}
					row++;
				}
			}
			reader.close();
		}
		catch (IOException ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}

	public static void main(String[] args) {
		TSP tsp = new TSP();
		System.out.println("*** Static test ***");
		Permutation spath = tsp.shortestPathStatic();
		int length = tsp.pathLength(spath);
		System.out.println("-> Shortest path: " + spath + " with length " + length);
		System.out.println("*** Dynamic test ***");
		tsp.readGraph("graph.txt");
		spath = tsp.shortestPath();
		length = tsp.pathLength(spath);
		System.out.println("-> Shortest path: " + spath + " with length " + length);
	}
}
